.packageName <- "aRT"
#  aRT : API R - TerraLib                                                
#  Copyright (C) 2003-2005  LEG                                          
#                                                                        
#  This program is free software; you can redistribute it and/or modify  
#  it under the terms of the GNU General Public License as published by  
#  the Free Software Foundation; either version 2 of the License, or     
#  (at your option) any later version.                                   
#                                                                        
#  This program is distributed in the hope that it will be useful,       
#  but WITHOUT ANY WARRANTY; without even the implied warranty of        
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
#  GNU General Public License for more details.                          
#                                                                        
#  You should have received a copy of the GNU Lesser General Public      
#  License along with this library.

# aRT.R: calls C++ functions of main.cpp.
# All this functions only check for user argument errors.
# Other errors are checked in the next layers.

aRT <- function()
{
	ver <- packageDescription("aRT", field="Version")
	dat <- packageDescription("aRT", field="Date")
	paste(ver, dat)
} 

# TEST FUNCTION
pp <-function(value)
{
    .Call("cppPP", value, PACKAGE="aRT")
    return(invisible())
}

# set the silent mode
aRTsilent <- function(value = NULL)
{
    .Call("cppSilent", value, PACKAGE="aRT")
}

######################################################################
#                          CLASSES DEFINITIONS                       #
######################################################################
setClass("aRTcomponent", representation(pointer = "externalptr", "VIRTUAL"))
setClass("aRT",      contains = "aRTcomponent")
setClass("aRTdb",    contains = "aRTcomponent")
setClass("aRTlayer", contains = "aRTcomponent")
setClass("aRTtheme", contains = "aRTcomponent")

#TODO: define representation classes (sp?)
#setClass("aRTid", representation=())
#setClass("aRTgeometry", representation

#setClass("aRTpoint",   contains = "aRTgeometry")
#setClass("aRTpolygon", contains = "aRTgeometry")
#setClass("aRTline",    contains = "aRTgeometry")

#setClass("aRTdata", presentation(data="aRT))

######################################################################
#                             CONSTRUCTORS                           #
######################################################################

# "destructor" function
aRTremove <- function(pointer)
{
	.Call("cppRemove", pointer, PACKAGE="aRT")
	return(invisible())
}

setMethod("initialize", "aRT", function(.Object,
                                        user = system("echo $USER", TRUE),
										password = "",
										port = 3306,
										host = "localhost",
										dbms = NULL)
{
  if(is.null(dbms))
  {
    if(FALSE) dbms = "postgre"
    if(TRUE)   dbms = "mysql"
  }

  if(dbms == "mysql"  & !TRUE)   stop("MySQL DBMS not found in the installation")
  if(dbms == "posgre" & !FALSE) stop("PostgreSQL DBMS not found in the installation")
  
  .Object@pointer <- .Call("cppNewaRT", user, password, port, host, dbms, PACKAGE="aRT")
  reg.finalizer(.Object@pointer, aRTremove)
  .Object
})

setMethod("initialize", "aRTdb", function(.Object,
                                          aRT,
										  dbname,
										  create = FALSE,
										  update = FALSE)
{
  if (!is.object(aRT) || class(aRT) != "aRT")
    stop("Argument \"aRT\" must be an object of aRT\n")
  if (is.null(dbname)) stop("dbname can not be NULL")

  out <- .Call("cppOpen",
               aRT@pointer,
			   list(dbname=dbname,
			        create=as(create, "integer"),
			        update=as(update, "integer")),
			   PACKAGE="aRT")

  if (is.null(out))
    stop("Could not create the aRTdb object!\n")
  .Object@pointer <- out
  reg.finalizer(.Object@pointer, aRTremove)
  .Object
})

setMethod("initialize", "aRTlayer", function(.Object,
                                             aRTdb,
											 layername,
											 create = FALSE,
											 proj = c("plan", "geographic"))
{
  if (!is.object(aRTdb) || class(aRTdb) != "aRTdb")
    stop("Argument \"aRTdb\" must an object of aRTdb\n")
  if (is.null(layername)) stop("layername can not be NULL")
  proj <- match.arg(proj)

  out <- .Call("cppOpen",
               aRTdb@pointer,
			   list(layername=layername,
			        create=as(create, "integer"),
					projection=proj),
			   PACKAGE="aRT")

  if (is.null(out))
    stop("Could not create the aRTlayer object!\n")
  .Object@pointer <- out
  reg.finalizer(.Object@pointer, aRTremove)
  .Object
})

setMethod("initialize", "aRTtheme", function(.Object,
                                             layer = NULL,
                                             db = NULL,
											 themename = summary(layer)$layer,
											 create = FALSE,
											 tables = NULL,
											 colors = terrain.colors(10),
											 viewname = themename)
{
# if tables == NULL the theme can't be created
# tables == showTables(aRTlayer) does not warrant that the theme will be opened,
# because TerraLib has some invalid table combinations (TODO: put it into help)
# tables and viewname are needed only when a new theme is going to be created
  if (is.null(themename)) stop("themename can not be NULL")

  if (is.object(layer) && class(layer) == "aRTlayer")
  {
	colors <- col2rgb(colors)
	if ( is.null(tables) && summary(layer)$raster == "no") tables = showTables(layer)

    out <- .Call("cppOpen",
                 layer@pointer,
				 list(themename = themename,
	  		          create = as(create, "integer"),
			          viewname = viewname,
			          tables = tables,
				      colors = colors),
				 PACKAGE="aRT")
  }
  else if(is.object(db) && class(db) == "aRTdb")
	  out <- .Call("cppOpen",
					db@pointer,
					list(themename = themename),
					PACKAGE="aRT")

  if (is.null(out)) stop("Could not create the aRTtheme object!\n")

  .Object@pointer <- out
  reg.finalizer(.Object@pointer, aRTremove)
  .Object
})

######################################################################
#                                PRINTING                            #
######################################################################

setGeneric("summary",    function(object, ...)  standardGeneric("summary"))
setGeneric("showDbs",    function(object)       standardGeneric("showDbs"))
setGeneric("showLayers", function(object)       standardGeneric("showLayers"))
setGeneric("showTables", function(object)       standardGeneric("showTables"))
setGeneric("showViews",  function(object)       standardGeneric("showViews"))
setGeneric("showThemes", function(object)       standardGeneric("showThemes"))
setGeneric("createEmptyTable", function(object) standardGeneric("createEmptyTable"))

setGeneric("createView", function(object, viewName)      standardGeneric("createView"))
setGeneric("addUser",    function(object, userName)      standardGeneric("addUser"))

setMethod("show", "aRTcomponent", function(object) {
	   .Call("cppPrint", object@pointer, PACKAGE="aRT")
	   return (invisible())
})

setMethod("addUser", "aRT", function(object, userName) {
	   .Call("cppAddUser", object@pointer, userName, PACKAGE="aRT")
	   return (invisible())
})

setMethod("summary", "aRTcomponent", function(object, ...) {
	   result <- .Call("cppSummary", object@pointer, PACKAGE="aRT")
	   return(result)
})

setMethod("showDbs", "aRT", function(object) {
	result <- .Call("cppList", object@pointer, PACKAGE="aRT")
	return(result)
})

setMethod("showLayers", "aRTdb", function(object) {
	result <- .Call("cppList", object@pointer, PACKAGE="aRT")
	return(result)
})

setMethod("showTables", "aRTlayer", function(object) {
	result <- .Call("cppShowTables", object@pointer, PACKAGE="aRT")
	return(result)
})

setMethod("createEmptyTable", "aRTlayer", function(object) {
	result <- .Call("cppCreateEmptyTable", object@pointer, NULL, PACKAGE="aRT")
	return(result)
})

setMethod("showViews", "aRTdb", function(object) {
	result <- .Call("cppShowViews", object@pointer, PACKAGE="aRT")
	return(result)
})

setMethod("showThemes", "aRTdb", function(object) {
	result <- .Call("cppShowThemes", object@pointer, PACKAGE="aRT")
	return(result)
})

setMethod("createView", "aRTdb", function(object, viewName) {
	out <- .Call("cppCreateViewFromDb", object@pointer, viewName, PACKAGE="aRT")
	return(invisible())
})

# still using??
#setGeneric("selectTables", function(object, tableNames) standardGeneric("selectTables"))

#setMethod("selectTables", "aRTtheme", function(object, tableNames) {
#	out <- .Call("cppSelectTables", object@pointer, tableNames)
#	return(invisible())
#})

setGeneric("importShape", function(object, layerName, fileName, proj = NULL) standardGeneric("importShape"))

setMethod("importShape", "aRTdb", function(object, layerName, fileName, proj = NULL) {
  out <- .Call("cppImportShape", object@pointer, layerName, fileName, proj, PACKAGE="aRT")
  if (is.null(out))
    stop("Invalid argument of aRTdb Class\n")
  newLayer <- new("aRTlayer", object, layerName)
  return(newLayer)
})

#setGeneric("populateLayer", function(object, data) standardGeneric("populateLayer"))

#setMethod("populateLayer", "aRTlayer", function(object, data, coords.col=1:2) {
## checking input
#    if (!is.data.frame(data)) stop("Spatia data must be a data.frame\n");
#    if (length(data)) < 3 stop("Spatial data must have two columns of coordinates and one column of data");
     #coords <- data[,coords.col];
     #res <- apply(coords, 2,  is.numeric)
#	out <- .Call("cppPopulateLayer", data, object@pointer);
#	return(out);
#})

# depois implementar append=TRUE
#setGeneric("insertNewAttr", function(object, data) standardGeneric("insertNewAttr"))

#setMethod("insertNewAttr", "aRTlayer", function(object, data) {
#	out <- .Call("cppInsertNewAttr", object@pointer, data)
#	return(out)
#})

######################################################################
#                             PLOTING                                #
######################################################################
setGeneric("plot",  function(x,y, ...) standardGeneric("plot"))
setMethod("plot", "aRTtheme", function(x, y, ...) {
  out <- getGeometry(x)
  plot(out, ...)
  return(invisible())
})

setMethod("plot", "aRTlayer", function(x, y, ...) {
  if( summary(x)$raster == "yes" )
  {
  	out <- getRaster(x)
	image(out, asp=1, ...)
  }
  else
  {
    out <- getGeometry(x)
    plot(out, ...)
  }
  return(invisible())
})

######################################################################
#                              REMOVING                              #
######################################################################

# FROM DATABASE
setGeneric("deleteDb",    function(object, dbname)                  standardGeneric("deleteDb"))
setGeneric("deleteTheme", function(object, themename)               standardGeneric("deleteTheme"))
setGeneric("deleteLayer", function(object, layername, confirm=TRUE) standardGeneric("deleteLayer"))
setGeneric("deleteView",  function(object, viewname)                standardGeneric("deleteView"))

setMethod("deleteDb", "aRT", function(object, dbname) {
	if(! any(showDbs(object) == dbname) ) stop("Database does not exist")

	.Call("cppDeleteDb", object@pointer, dbname, PACKAGE="aRT")
	return(invisible())
})

setMethod("deleteTheme", "aRTdb", function(object, themename) {
    .Call("cppDeleteTheme", object@pointer, themename, PACKAGE="aRT")
	return(invisible()) 
})

# TODO: confirm
#setGeneric("ttt", function(object) standardGeneric("ttt"))
#setMethod("ttt", "aRT", function(object) {
#	c <- "a"
#	while(c != "y" && c != "n")
#	{
#		cat("Do you really want to remove permanently the layer? [y/n]: ")
#		c <- .Call("cppConfirm")
#		cat("teste")
#		
#	}
#	return(invisible())
#})

setMethod("deleteLayer", "aRTdb", function(object, layername, confirm=TRUE) {
#	if(confirm)
#	{
#		cat("Do you really want to delete the layer", layername, "? (y,n)")
#		scan(file="", what=character(0), n=1, quiet=TRUE);
#	}
#	cat("Voce digitou",a)

    .Call("cppDeleteLayer", object@pointer, layername, PACKAGE="aRT")
	return(invisible())
})

setMethod("deleteView", "aRTdb", function(object, viewname) {
    .Call("cppDeleteView", object@pointer, viewname, PACKAGE="aRT")
	return(invisible())
})

######################################################################
#                          THEME RESTRICTIONS                        #
######################################################################
#setGeneric("setAR", function(object, restriction) standardGeneric("setAR"))
#setGeneric("setTR", function(object, restriction) standardGeneric("setTR"))
#setGeneric("setSR", function(object, restriction) standardGeneric("setSR"))

#setMethod("setAR", "aRTtheme", function(object, restriction) {
#	out <- .Call("cppSetAR", object@pointer, restriction, PACKAGE="aRT")
#	return(invisible())
#})

#setMethod("setTR", "aRTtheme", function(object, restriction) {
#	out <- .Call("cppSetTR", object@pointer, restriction, PACKAGE="aRT")
#	return(invisible())
#})

#setMethod("setSR", "aRTtheme", function(object, restriction) {
#	out <- .Call("cppSetSR", object@pointer, restriction, PACKAGE="aRT")
#	return(invisible())
#})

######################################################################
#                    ENVIRONMENT OF A DATABASE                       #
######################################################################

setGeneric("saveEnv",   function(object, filename) standardGeneric("saveEnv"))
setGeneric("loadEnv",   function(object, filename) standardGeneric("loadEnv"))
setGeneric("getEnv",    function(object)           standardGeneric("getEnv"))
setGeneric("deleteEnv", function(object)           standardGeneric("deleteEnv"))

setMethod("saveEnv", "aRTdb", function(object, filename){
	vect_view  <- numeric(0)
	vect_layer <- numeric(0)
	vect_theme <- numeric(0)
	vect_table <- numeric(0)
	i <- 1
	n_theme <- showThemes(object)

	if (length(n_theme > 0)) {
    	for (k in 1:length(n_theme)) {
        	theme <- new("aRTtheme", db=object, themename=as.character(n_theme[k]))
	       	s_theme <- summary(theme)
	        for(j in 1:length(names(s_theme$tables))) {
		   		vect_theme[i] <- s_theme$theme
			    vect_view[i]  <- s_theme$view
			    vect_layer[i] <- s_theme$layer
			    vect_table[i] <- names(s_theme$tables[j])
				i <- i+1
			}
		}
	}
	else
	  return(invisible())
    
	data <- data.frame(View=vect_view, Layer=vect_layer, Theme=vect_theme, Table=vect_table)
	write.table (data, file=filename, quote=FALSE, sep=" ", row.names=FALSE)
})

setMethod("loadEnv", "aRTdb", function(object, filename) {
  deleteEnv(object)
  data <- read.table(filename, header=TRUE)
  for(i in 1:nrow(data)){
    layer <- new("aRTlayer", object, l=as.character(data$Layer[i]))

	if(data$Table[i] == "_all_")
	{
		if(summary(layer)$raster == "no") tables_layer <- showTables(layer)
		else							  tables_layer <- NULL
	}
	else
		tables_layer <- as.character(data$Table[data$Theme == as.character(data$Theme[i])])

    if (data$Theme[i] != data$Theme[i+1] || i == nrow(data)) {
       createView(object, as.character(data$View[i]))
       theme <- new("aRTtheme",
                    layer,
                    themename=as.character(data$Theme[i]),
                    create=TRUE,
                    tables=tables_layer,
                    viewname=as.character(data$View[i]))
    }
  }			 
})

setMethod("getEnv", "aRTdb", function(object){
	vect_view  <- numeric(0)
	vect_layer <- numeric(0)
	vect_theme <- numeric(0)
	vect_table <- numeric(0)
	i <- 1
	n_theme <- showThemes(object)

	if (length(n_theme > 0)) {
   		for (k in 1:length(n_theme)) {
        	theme <- new("aRTtheme", db=object, themename=as.character(n_theme[k]))
		    s_theme <- summary(theme)
			for(j in 1:length(names(s_theme$tables))) {
				vect_theme[i] <- s_theme$theme
				vect_view[i]  <- s_theme$view
				vect_layer[i] <- s_theme$layer
				vect_table[i] <- names(s_theme$tables[j])
				i <- i+1
			}
		}
	}

	data <- data.frame(View=vect_view, Layer=vect_layer, Theme=vect_theme, Table=vect_table)
	return(data)
})

setMethod("deleteEnv", "aRTdb", function(object){
  views <- showViews(object)
  if( !is.null(views) )
  for(i in 1:length(views))
  	deleteView(object, as.character(views[i]))  
})

######################################################################
#                            INSERTING DATA                          #
######################################################################
setGeneric("addPoints",   function(object, points, close=T)       standardGeneric("addPoints"))
setGeneric("addPolygons", function(object, polygons, close = T)   standardGeneric("addPolygons"))
setGeneric("addLines",    function(object, lines, close = T)      standardGeneric("addLines"))
setGeneric("addRaster",   function(object, value)                 standardGeneric("addRaster"))
setGeneric("addColumns",  function(object, data)                  standardGeneric("addColumns"))

setMethod("addPoints", "aRTlayer", function(object, points, close = T) {
	if( !any(names(points)=="id"))     stop("points must have a list called 'id'")
	if( !any(names(points)=="points")) stop("points must have a list called 'points'")
	if( length(points$points) != length(points$id))
	    stop("'id' must have the same length of 'points'")
		
	.Call("cppAddPoints", object@pointer, points, PACKAGE="aRT")
	if(close) .Call("cppCreateEmptyTable", object@pointer, PACKAGE="aRT")
	return(invisible())
})

setMethod("addPolygons", "aRTlayer", function(object, polygons, close = T) {
    if( !any(names(polygons)=="id"))       stop("polygons must have a list called 'id'")
    if( !any(names(polygons)=="polygons")) stop("polygons must have a list called 'polygons'")
	if( length(polygons$polygons) != length(polygons$id))
	    stop("'id' must have the same length of 'polygons'")

	.Call("cppAddPolygons", object@pointer, polygons, PACKAGE="aRT")
	if(close) .Call("cppCreateEmptyTable", object@pointer, PACKAGE="aRT")
	return(invisible())
})

setMethod("addLines", "aRTlayer", function(object, lines, close = T) {
    if( !any(names(lines)=="id"))       stop("lines must have a list called 'id'")
	if( !any(names(lines)=="lines"))    stop("lines must have a list called 'lines'")
	if( length(lines$lines) != length(lines$id))
	    stop("'id' must have the same length of 'lines'")

	.Call("cppAddLines", object@pointer, lines, PACKAGE="aRT")

    if(close) .Call("cppCreateEmptyTable", object@pointer, PACKAGE="aRT")
	return(invisible())
})

# receives as arguments n>1 lists of data. one of them must be $id
setMethod("addColumns", "aRTlayer", function(object, data) {
	if( !any(names(data)=="id") ) stop("data must have a list called 'id'")
	if( !is.character(data$id) )  stop("'id' must be in character format")

	if( !is.null(data$points)   ) data$points   <- NULL
	if( !is.null(data$polygons) ) data$polygons <- NULL
	if( !is.null(data$lines)    ) data$lines    <- NULL
    # other geometries...

	other<-names(data[-which(is.element(names(data),"id"))])

	for(i in 1:length(other))
	{
		size<-0
		data_ <- data[[other[i]]]

		if(is.character(data_[1]))
		{
			type<-"s"
	        for(j in 1:length(data))
	        {
	            if(nchar(data_[j]) > size)
	                size <- nchar(data_[j])
	        }
	    }
		else if(is.real(data_[1]))
			type<-"r"
		else if(is.integer(data_[1]))
			type<-"i"
		else stop(paste("unknown type of ", other[i]))
	
		if( length(data$id) != length(data_) )
			stop("data with different lengths")
	
		config<-data
		config$type_<-type
		config$name_<-other[i]
		config$size_<-size
	
	    .Call("cppAddColumn", object@pointer, config, PACKAGE="aRT")
	}
    return(invisible())
})

# create and populate a table
#setMethod("addTable", "aRTlayer", function(object, tablename, table, key) {
	# ... built the description and insert data in the database
    # percorrer table para descobrir os atributos a serem colocados em mydescription
	# e entao criar a tabela. se algum campo for string, mydescription$size tem que pegar
	# o tamanho da maior string da coluna. checar os tipos e colocar em $type
#	att<-NULL
#	att$name<-names(table)
#	att$size<-NULL
#	att$type<-NULL
#	for(i in 1:length(table))
#    {
#		bigger<-0
#		if(is.character(table[[i]][1]))
#		{
#			att$type<-append(att$type,"s")
#			for(j in 1:length(table[[i]]))
	#		{
		#		if(nchar(table[[i]][j]) > bigger)
			#		bigger <- nchar(table[[i]][j])
	#		}
	#	}
	#	else if(is.real(table[[i]][1]))
	#	{
	#		att$type<-append(att$type,"s")
	#	}
	#	else if(is.integer(table[[i]][1]))
    #    {
    #        att$type<-append(att$type,"i")
    #    }
	#	else
	#		error("invalid type")
	#	att$size<-append(att$size, bigger)
	#}
	#createTable(object, tab=tablename, desc=att, k=key)
	#addElements(object, tablename, table)
	#return(att)
#})

# add new elements to a existent layer
#setMethod("addElements", "aRTlayer", function(object, tablename, table) {
#	if(!is.data.frame(table))    stop("table must be a dataframe")
#	.Call("cppAddElements", object@pointer, tablename, table$attributes)
#	return (invisible())
#})

# this function creates a table, but not populates it. For this use addElements.
# description$type must be one of (for each item):
# "s": string
# "d", "i": integer
# "f", "r": real
#setMethod("createTable", "aRTlayer", function(object, tablename, description, key) {
# TODO: mudar para data.frame
#    if(!is.data.frame(description))          stop("description must be a data.frame")
#	if(!any(names(description)=="name"))  stop("description must have a slot named name")
#	if(!any(names(description)=="type"))  stop("description nust have a slot named type")
#    if(!any(description$name==key))       stop("key must be one of the names of table")
#	if(!any(description$type=="s") && !any(names(description)=="size"))
#		stop("if there are string attributes, description must have slot named size")
	#  -1 because the R =>C vectors
#	print(which(description$name==key))
#	.Call("cppCreateTable",object@pointer, tablename, description, which(description$name==key)-1)
#	return (invisible())
#})

setMethod("addRaster", "aRTlayer", function(object, value) {
  if(summary(object)$raster == "yes") stop("the layer already has raster data")
  if(!any(names(value)=="z"))         stop ("'value' must have a z attribute")
  if(!any(names(value)=="x"))         stop ("'value' must have a x attribute")
  if(!any(names(value)=="y"))         stop ("'value' must have a y attribute")

  .Call("cppAddRaster",
        object@pointer,
        list(x=range(value$x),
             y=range(value$y),
             z=value$z,
             dim = dim(value$z)),
		PACKAGE="aRT")

  return(invisible())
})


######################################################################
#                            DATABASE ACCESS                         #
######################################################################
setGeneric("getBox",        function(object) standardGeneric("getBox"))
setGeneric("getGeometry",   function(object) standardGeneric("getGeometry"))
setGeneric("getData",       function(object) standardGeneric("getData"))
setGeneric("getAttributes", function(object) standardGeneric("getAttributes"))
setGeneric("getRaster",     function(object) standardGeneric("getRaster"))

setGeneric("getDistance",   function(object, layer=object, id) standardGeneric("getDistance"))
setGeneric("getNN",         function(object, layer=object, id, quantity=1) standardGeneric("getNN"))
setGeneric("getMetric",     function(object, metric, distance=NULL) standardGeneric("getMetric"))
setGeneric("getSetMetric",  function(object, metric, id) standardGeneric("getSetMetric"))
setGeneric("getRelation",   function(object, relation, layer=object, id=NULL) standardGeneric("getRelation"))

setMethod("getBox", "aRTlayer", function(object) {
  .Call("cppGetBox", object@pointer, PACKAGE="aRT")
})

setMethod("getNN", "aRTlayer", function(object, layer=object, id, quantity=1) {
	.Call("cppGetNearestNeighbors",
		  object@pointer,
		  list(id=id,
		       quantity=quantity),
		  layer@pointer,
		  PACKAGE="aRT")
})

setMethod("getDistance", "aRTlayer", function(object, layer=object, id) {
	.Call("cppGetDistance",
	      object@pointer,
		  id,
		  layer@pointer,
		  PACKAGE="aRT")
})

setMethod("getRelation", "aRTlayer", function(object, relation, layer=object, id=NULL) {
	.Call("cppGetRelation",
          object@pointer,
          list(relation=relation,
               id=id),
		  layer@pointer,
          PACKAGE="aRT")
})

setMethod("getSetMetric", "aRTlayer", function(object, metric, id) {
  result <- .Call("cppGetSetMetric",
                  object@pointer,
                  list(metric=metric,
                       id=id),
                  PACKAGE="aRT")

  if( !is.null(result) )
    class(result)<-"aRTgeometry"

  result
})

setMethod("getMetric", "aRTlayer", function(object, metric, distance=NULL) {
  result <- .Call("cppGetMetric",
                  object@pointer,
                  list(metric=metric,
                       dist=distance),
                  PACKAGE="aRT")

  if( any(metric == c("h", "c", "b")) )
    class(result)<-"aRTgeometry"

  result
})

#setMethod("getBox", "aRTtheme", function(object) {
#  .Call("cppGetBox", object@pointer, PACKAGE="aRT")
#})

setMethod("getGeometry", "aRTtheme", function(object) {
   result <- .Call("cppGetThemeGeometry", object@pointer, PACKAGE="aRT")
	if(is.null(result)) stop("Error loading data");
   class(result)<-"aRTgeometry"
   result
})

setMethod("getGeometry", "aRTlayer", function(object) {
	result <- .Call("cppGetLayerGeometry", object@pointer, PACKAGE="aRT")
	if(is.null(result)) stop("Error loading data");
    class(result)<-"aRTgeometry"
    result
})

setMethod("getData", "aRTlayer", function(object) {
	result <- .Call("cppGetLayerData", object@pointer, PACKAGE="aRT")
    if(is.null(result)) stop("Error loading data");
	class(result)<-"aRTgeometry"
	result
})

setMethod("getData", "aRTtheme", function(object) {
  .Call("cppGetThemeData", object@pointer, PACKAGE="aRT")
})

setMethod("getAttributes", "aRTtheme", function(object) {
  .Call("cppGetThemeAttributes", object@pointer, PACKAGE="aRT")
})

setMethod("getAttributes", "aRTlayer", function(object) {
  .Call("cppGetLayerAttributes", object@pointer, PACKAGE="aRT")
})

setMethod("getRaster", "aRTlayer", function(object) {
  .Call("cppGetRaster", object@pointer, PACKAGE="aRT")
})


.First.lib <- function(lib, pkg) {
    library.dynam("aRT", pkg, lib)
}

